
# q-expansion of newform 6468.2.a.be, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 6468
weight = 2
poly_data = [7, 56, 44, -18, -18, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[305, 525, -101, -149, 1, 7], 162], [[277, -138, -139, 62, 11, -4], 81], [[67, -21, -55, 11, 5, -1], 18], [[605, 2595, 121, -1079, -83, 67], 162], [[-605, -2271, -121, 1079, 83, -67], 162]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 0], [-1, 0, 0, 1, 0, 0], [0, 0, 0, 0, 0, 0], [-1, 0, 0, 0, 0, 0], [-1, 0, 0, 0, -1, 0], [-2, 1, 0, -1, 0, 1], [-1, -2, -1, 0, -1, -1], [0, 1, 1, -1, 1, -1], [-1, -2, -1, -1, 1, 0], [0, 1, -1, -1, 1, 1], [1, 2, -1, 0, -1, 1], [-2, 1, 2, 0, 1, -1], [0, 0, 2, -1, 0, 1], [3, 0, 3, -1, 1, 0], [0, 1, -1, 1, -1, -1], [-5, 2, -3, 1, 0, 1], [-2, -4, 1, -2, 1, 0], [1, 0, -1, 0, 0, 0], [-6, -3, -1, 0, 0, 1], [-7, 2, -2, 2, -2, -1], [0, 2, -2, -2, 0, 0], [-6, -1, -1, 1, 1, 1], [-4, -4, -3, 2, 0, -1], [-4, 2, 1, 1, 1, -3], [-6, 1, 2, 1, 0, 1], [4, -3, 5, 0, -1, -2], [-5, -2, 1, 3, -4, -1], [0, -1, -1, 1, -3, -1], [4, 1, 3, -2, 0, 3], [-2, -2, 2, 3, 0, -1], [-6, -1, -1, 2, -2, 1], [-2, 5, 3, -1, 1, -1], [-2, -6, 0, -2, -1, 1], [1, 0, 1, 1, 3, -2], [0, 9, 1, -1, 1, -1], [-4, 2, 1, -4, 1, 0], [7, 0, 1, 0, 2, 2], [-4, 1, 1, 3, 1, -1], [0, 6, -1, 0, 1, 2], [0, 4, -4, 4, 0, 0], [-8, -2, -1, -3, 3, 1], [-8, 2, 2, 3, 1, 0], [2, 3, -1, 1, 1, 3], [4, 0, -4, 0, -3, 1], [0, 4, 4, -5, 2, 1], [2, -8, -2, 0, 0, 0], [2, 0, -4, 0, 0, 2], [-8, -1, 1, 2, 0, -1], [2, -3, -2, -3, 2, 3], [0, 4, 2, -2, 1, -1], [-1, -8, -1, 1, 0, -1], [-7, -2, 2, 0, 5, -2], [-5, 6, 1, 1, -2, -1], [-3, 2, 6, -3, 2, -2], [5, 0, 3, -3, 3, 2], [-6, -6, 1, 0, -1, -4], [9, 6, 1, -2, 4, 0], [6, -9, 1, 2, -3, -2], [1, -2, -1, 3, 2, 1], [-5, 8, -1, 0, 0, -2], [0, -8, -1, -2, 1, 0], [-6, 2, 6, 0, 0, 0], [-2, -6, 4, 4, -1, -1], [-4, 1, 2, 3, 2, -1], [-6, -2, -2, 3, -5, -2], [4, 2, -2, 6, 0, -4], [2, 1, 3, -5, 1, 3], [-4, 0, -2, 2, 1, -1], [-3, -6, 0, 4, -4, 1], [-15, -4, -2, 1, -5, 1], [-6, 2, -2, -2, 3, -1], [-2, 4, -4, 4, 2, 0], [6, 2, -4, -1, -4, 3], [5, -4, 1, 4, -5, -1], [12, 8, 0, 2, -5, 3], [-6, -5, 5, -4, 0, -1], [-16, -7, 2, -3, -2, 1], [-10, -4, 2, -1, 3, 0], [-10, 2, 1, 2, -2, -5], [-7, -4, -5, 7, 1, -2], [5, -6, -3, 4, 1, -1], [-7, 2, -5, -1, 0, -1], [-4, -4, -5, -2, 3, 2], [-7, -2, -3, 6, 0, 0], [2, -1, -3, 0, 2, 1], [2, -7, -7, 2, -6, 1], [2, 9, -3, 7, -1, 1], [-10, 3, -2, -2, -1, -5], [1, 0, -3, 2, 1, 3], [3, -8, -1, -1, 2, -1], [0, 2, 2, -1, 1, 4], [22, 2, 4, -4, -2, 0], [-1, -8, 3, -1, -6, -1], [7, -6, 3, 4, 0, -2], [6, -2, -4, -5, 3, 6], [-10, 2, -5, 6, -4, -1], [-1, -2, 0, -7, 2, 6], [-15, 0, 7, 2, -1, -5], [22, 5, -1, -2, -1, 2], [10, 1, 9, -2, 3, 2], [5, -12, 1, 3, -1, -4], [-2, 14, 6, -5, 7, 4], [-6, 2, 2, 0, -3, -1], [10, 7, -5, 6, -3, 2], [-4, 3, -2, 1, 4, 7], [1, -2, 5, -7, -1, 4], [-22, 1, 0, -8, 3, -1], [-12, 6, -2, 4, -2, -4], [13, -10, 4, 4, 1, -4], [1, 10, 1, -4, -2, 4], [-2, 10, 2, 3, 0, -1], [2, 4, -2, 8, 2, -2], [-2, 2, -6, -1, -6, 1], [-4, 2, -6, 6, 0, 4], [-2, 1, 3, 3, -5, 1], [-8, -12, 2, 1, -4, -5], [9, -16, -7, 3, -5, -2], [-12, 2, -10, 6, 2, -2], [-3, 0, -3, -7, 6, 1], [-10, 19, -8, -5, -2, 3], [-4, 12, -6, -5, -6, 3], [-12, 3, 8, 0, 1, -5], [-4, 4, 4, -7, 1, 4], [22, 0, -6, 8, -6, 2], [6, 4, -2, 2, 2, 8], [13, 6, -1, 0, -1, -3], [-3, -12, -1, -1, -6, -7], [6, 3, -5, -3, -1, 3], [-4, -18, -5, 2, -3, 0], [-2, 3, 1, -5, 7, 7], [1, 8, 3, 1, -1, 2], [17, 0, 3, 4, -8, -4], [9, 2, 5, -4, -2, 0], [-4, 2, -1, -8, 3, 4], [-19, 4, -6, 8, -2, -3], [-3, -6, -2, -1, 1, 5], [-7, 8, -3, 4, 4, 2], [11, 16, -4, 1, -5, 1], [-1, -14, 7, 5, 0, -3], [-1, -12, -1, 6, 0, 0], [11, 4, -5, -3, 1, 4], [3, 16, -9, 0, -3, 5], [-15, 8, 7, -3, 3, -2], [4, 24, 1, -4, -1, 2], [13, 4, -3, 1, -3, 2], [-6, 2, 5, 4, 5, 2], [-6, -12, 7, -12, 1, 0], [-4, 0, 4, -6, 0, -2], [-14, -4, -2, -1, -5, 0], [-8, -4, 4, 0, 8, 0], [5, 4, -4, -9, 3, 1], [-1, 2, -11, -2, -1, 3], [-4, -2, -10, 1, 3, -4], [10, 2, -6, 12, -7, -1], [-4, -13, 7, -6, 2, -1], [10, 9, 3, 3, -3, 3], [15, -2, 4, -3, -7, -5], [0, 4, -5, 2, -5, 6], [0, -4, -5, 0, -3, 2], [-18, -2, 0, 4, 2, 4], [5, 8, 3, -3, 6, -7], [24, 3, 5, -4, -3, -4], [23, 4, -1, 1, -4, 3], [0, 3, 3, -7, 1, 3], [6, 12, 2, 2, -3, -3], [3, -12, 3, 0, -3, -3], [0, 20, 3, -2, 0, 1], [-28, 3, 3, 4, 3, -2], [-29, -8, 0, -1, 3, -5], [-27, 6, -3, 1, -3, -2], [14, 8, 7, -6, 9, 6], [-2, -6, 0, 2, 0, 4], [8, -23, 5, 7, 1, -1], [18, 8, 4, -2, 0, 0], [16, 0, -9, -8, 6, 5], [2, 3, -3, -1, 1, -7], [4, 1, 5, 0, -2, -1], [1, 0, -1, -8, 0, 4], [-23, 0, 0, 2, 3, 0], [1, -6, -13, 8, -9, 1], [-12, -7, -1, 7, -3, -3], [-22, 2, 4, 6, 0, -4], [10, 17, 2, 4, -5, -3], [16, -4, -4, -4, -2, -2], [27, 14, 6, -3, 0, -2], [-8, -20, -6, 8, -6, 6], [14, -8, -4, -7, 0, -1], [-2, 0, 0, 11, 4, -7], [6, -17, 9, 1, -3, -5], [10, -14, -1, 4, -1, -2], [1, 0, -9, -1, 1, -2], [9, -26, -1, 0, -3, -1], [-15, 0, 4, 7, -4, 0], [13, -26, 7, 3, 0, -1], [15, -10, 0, -9, 1, 1], [0, -11, -3, -7, -3, 5], [-6, -20, 2, 2, -2, 0], [-8, -21, -4, 7, -8, -1], [4, -3, 1, -3, -3, 5], [-18, 0, 4, 10, 6, -4], [21, 2, -5, 6, 1, -3], [2, -7, 0, -5, -8, -3], [-23, 6, -8, 12, 2, -1], [4, 14, 8, -2, 1, -7], [-20, -2, -1, 4, 6, -3], [14, -5, 1, 8, -1, -4], [-17, -16, -1, 7, -6, 3], [-24, 18, 6, 0, 3, 1], [22, 7, 1, -3, -5, 5], [0, 11, -11, 0, 7, 2], [1, 6, 8, -7, 1, 5], [3, -6, 3, 6, 2, -6], [18, -8, -6, -4, -2, -2], [20, -3, 5, -6, 2, -3], [-10, -8, 1, 2, 1, 0], [10, 7, 7, -2, 1, 2], [18, 5, -6, -5, -10, 5], [2, 1, -1, -5, 11, -3], [-12, 10, -2, -5, 1, 0], [-10, 0, -4, 3, 2, -1], [13, -10, 11, -2, 1, -3], [0, -1, -7, 3, -13, -1], [15, 0, -7, 8, 2, -2], [4, 8, 2, -6, 4, 4], [-24, 13, -1, 4, -1, 0], [-11, 12, -4, -5, 1, -1], [10, 16, -6, -8, 4, -4], [-11, -22, -3, 10, 0, 0], [-25, -6, -3, 1, -6, -9], [8, -10, 8, 0, 1, 3], [3, -8, -7, 4, -5, -3], [0, -25, 5, -4, -7, -8], [-13, 10, 3, -3, 10, 7], [14, -11, -5, 3, -7, -9], [-12, -3, -15, 8, -4, -3], [14, -2, 9, -17, 5, 1], [35, -8, 9, -1, -2, -1], [14, 9, 3, -5, 7, 9], [-7, 12, -7, 7, -7, -4], [-16, 15, -1, 3, 5, 5], [-12, 12, -6, 0, 2, -4], [34, -2, 4, -7, 2, 1], [0, -6, 6, 6, -1, 7], [-16, 8, -9, 8, -7, 2], [32, 4, -4, 4, 6, -6], [28, -16, 6, 4, 2, 0], [35, 0, 7, -3, 4, 3], [-2, -9, -5, 6, -5, 8], [29, 4, -5, 3, -1, 4], [-8, 13, 1, 8, 3, -12], [-18, 18, -5, -9, 4, 4], [22, -24, -2, -2, 0, 6], [-10, 2, -1, -10, 7, -4], [4, 15, -2, 8, 1, 1], [-26, 0, 4, 4, -3, 5], [-19, 4, -5, 2, -4, 2], [-1, 14, 9, -2, 8, 10], [1, 14, 0, 13, 3, -1], [-11, -10, 0, 0, -4, -3], [-2, -14, -4, 10, -12, -8], [6, 22, -4, 2, -2, 2], [16, 12, -9, 6, -5, 6], [11, 0, 8, -4, 6, 5], [-1, -4, -4, -7, -4, 8], [0, 23, 7, 3, 5, 1], [22, 16, 4, -2, 10, 4], [5, -10, 0, -15, 8, 6], [0, 14, 6, -8, 8, 2], [6, -18, 6, -4, -5, -3], [24, -4, -1, -12, -3, -2], [27, 22, 1, -4, 13, 3], [-22, -11, 2, -5, 0, -9], [2, 26, -6, -5, 4, -1], [15, 10, 6, -4, 6, -9], [-6, 0, -6, 9, -2, -5], [-4, -12, 0, 4, -2, -2], [-34, -20, -4, 6, -14, 0], [-18, 2, -12, 3, -10, 1], [0, -1, 5, 3, -5, -11], [28, 5, 5, -7, -3, 5], [-4, 36, 4, -4, 6, 2], [8, -8, -4, 3, -1, 4], [1, -28, -6, 2, 3, 2], [11, 4, 3, -2, -10, 2], [-26, 4, -10, 7, -5, -4], [-19, -8, 3, 6, -11, -1], [28, -6, 2, 8, -5, 1], [2, -6, -12, 3, -2, -1], [24, -26, 11, 9, -4, -8], [-2, 29, 1, 3, 3, -3], [4, -12, -6, 2, -11, 3], [-1, -4, 9, 9, 1, -12], [2, -6, 2, -9, 5, 10], [-36, -16, -6, 4, -8, -4], [-26, -4, -3, -9, 2, 0], [10, 22, 2, -4, 4, 0], [8, -12, 11, 6, 1, -8], [-14, 6, 18, -7, 7, -6], [-31, 12, -1, -14, 9, 9], [-21, -6, 6, 2, 7, -4], [17, 2, -7, -3, -9, -4], [4, 18, -8, -3, -12, 9], [2, 20, 6, 7, -11, -2], [17, -18, 15, 8, -1, 1], [1, 2, -1, -4, -9, -3], [2, -2, 0, -17, 5, 4], [12, -14, -11, 11, -11, -1], [-10, -2, 8, -13, -1, 2], [-22, 13, 1, 3, -5, -3], [0, 23, -1, 0, -2, 5], [12, -12, -8, 4, -12, -4], [-19, -12, -1, -5, 1, -6], [8, -4, -2, 0, 3, -5], [22, -4, -12, 8, -5, 7], [-50, -10, -3, 2, -2, -1], [-22, -4, -6, 0, 6, 2], [-16, -8, -4, 0, -10, 6], [18, 19, 14, -9, 2, -3], [10, 4, -14, -12, 8, 4], [17, -6, 5, -4, 9, 1], [1, -6, 1, -8, 4, 2], [18, -8, 7, -8, 19, 2], [18, -30, 2, 6, -1, -1], [-41, 8, -19, 7, -2, -1], [20, 4, -3, 12, 1, -12], [-8, 0, 10, -4, -1, 3], [16, -2, 12, 3, -2, -3], [-9, 4, 7, 11, -8, -3], [-11, -10, -17, 11, -12, -5], [-6, 13, 15, -6, -5, 10], [24, -18, 18, -4, 2, -4], [-9, 10, 9, -8, -3, 11], [-42, 19, -13, 3, -5, -7], [0, 16, -6, -15, 4, 3], [-29, 10, -7, -3, 0, 1], [-5, -8, 5, -2, 18, 0], [14, 22, -3, -5, -2, 0], [8, 13, 8, -9, 4, 3], [-38, 6, 8, -8, 8, -6], [-14, 10, -2, -10, 6, -6], [-2, -11, 7, -1, 7, 5], [-16, 10, -1, 5, -4, 0], [11, -6, -5, -16, 10, 8], [18, 11, 1, -9, 11, -1], [-5, 20, -1, -1, 2, -1], [-4, -11, -12, 8, -12, -4], [-2, 2, -10, 2, -7, 9], [20, -12, -8, -2, -8, 2], [-16, 4, 5, 4, 16, 5], [24, 20, 6, -6, 16, 0], [9, -20, 3, 12, -7, -7], [-22, 11, 7, -2, 6, 3], [1, -6, 5, -12, 0, 6], [6, 24, -2, -11, 2, 1], [-28, -6, 5, 5, -10, -8], [-27, 8, -1, -11, 10, -1], [8, 8, 6, -6, 6, 2], [9, 2, 5, -9, 12, -1], [-4, -17, 3, 5, -1, -7], [18, 3, 17, -16, 5, -4], [-40, 14, -3, 10, -7, -6], [18, -1, 5, 14, 2, 3], [15, -4, 13, 1, 7, -6], [-16, -10, 10, -8, -2, 0], [-15, -22, -9, -4, 11, 1], [6, 5, -16, -1, 8, 1], [-12, -7, -9, -4, 1, 2], [12, 3, 11, -10, -1, 16], [-44, 6, -6, 5, 3, 0], [20, -3, 1, 5, -13, -5], [-17, 10, 1, 17, -2, -9], [15, -10, 11, -1, -6, 1], [-40, -24, 0, 0, 4, 4], [-5, 18, 5, 2, 16, 4], [-1, 30, -5, 3, -5, -2], [5, -20, -15, 7, -12, -3], [9, 0, -4, 16, -4, 5], [-12, -43, 2, -4, -3, -3], [-52, 6, -3, 10, -5, -4], [-3, 10, -2, 15, -11, -7], [10, -26, 2, -13, -5, 6], [-4, 9, -1, 4, -5, 8], [-25, -24, -1, 1, 15, -6], [-24, 10, -12, -4, 6, 10], [-31, -14, 5, -5, 12, -3], [-3, 42, 9, 2, -4, 0], [-18, -8, 5, -16, 7, -4], [0, 15, 3, 13, -9, -7], [3, 0, 3, -5, 1, 4], [-3, -32, -13, 4, -8, 0], [44, 12, -1, 6, -6, 3], [-18, -4, -6, 1, -3, -4], [-52, 12, -6, 5, -2, -7], [-36, -1, -1, 6, 8, 1], [18, 2, 18, -3, 8, -3], [-5, -20, -7, -8, 14, 2], [-29, -26, -8, 7, -4, 6], [20, -26, -6, -2, -6, 6], [25, 6, -3, -1, -7, -8], [-45, 16, 6, -12, 7, -2], [12, -14, 8, -3, -3, 2], [0, 23, -9, 9, 5, 3], [6, -22, -3, 0, -7, 10], [-14, -14, 5, 5, -12, 8], [59, -2, 1, 6, 3, -1], [-23, 16, -3, 16, 0, -4], [10, -12, 12, 10, 4, -2], [26, 6, 4, -2, 6, -6], [-18, 10, -6, 6, -13, -5], [-14, -6, -7, -4, 13, 6], [-12, 8, -8, -2, -8, 6], [-14, 5, 3, 0, 0, 13], [-22, -13, -15, 15, -5, 3], [0, -19, 7, 12, 3, 4], [27, -14, -2, 1, -5, 11], [15, -22, 7, 13, -11, -10], [-14, -20, -10, 15, -18, 5], [36, 18, 11, -14, 22, -1], [-12, -16, 8, 11, -11, -2], [-38, -15, -5, -1, -3, -9], [-13, -6, 18, 6, 0, -17], [16, -6, 10, -21, 9, 4], [-26, 21, 5, -19, 7, 1], [2, 14, -13, 2, 5, 2], [11, -10, 12, 3, -4, -10], [20, 4, 16, -3, -7, -4], [-11, -18, -3, 7, -9, -2], [-8, -33, 1, 10, -5, -4], [26, 1, 7, -14, 16, -1]]
